==> copilot-24.7.3-sources.jar: com/vaadin/copilot/ide/IdePluginCommandHandler.java <==
package com.vaadin.copilot.ide;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import com.vaadin.base.devserver.DevToolsInterface;
import com.vaadin.copilot.CopilotCommand;
import com.vaadin.copilot.FlowUtil;
import com.vaadin.copilot.SpringBridge;
import com.vaadin.copilot.plugins.themeeditor.ThemeModifier;
import com.vaadin.flow.component.internal.ComponentTracker;
import com.vaadin.flow.internal.JsonUtils;

import elemental.json.JsonObject;
import elemental.json.JsonValue;

public class IdePluginCommandHandler extends CopilotCommand {

    private static final String UI_ID = "uiId";

    private final CopilotIDEPlugin plugin = CopilotIDEPlugin.getInstance();

    @Override
    public boolean handleMessage(String command, JsonObject data, DevToolsInterface devToolsInterface) {
        try {
            if (command.equals("plugin-undo")) {
                handlePluginUndoCommand(data);
                return true;
            }

            if (command.equals("plugin-redo")) {
                handlePluginRedoCommand(data);
                return true;
            }
            if (command.equals("plugin-restart-application")) {
                handlePluginRestartApplicationCommand();
                return true;
            }

            if (command.equals("plugin-refresh") && plugin.supports(CopilotIDEPlugin.Commands.REFRESH)) {
                handlePluginRefreshCommand();
                return true;
            }

        } catch (IOException e) {
            throw new UnsupportedOperationException("Copilot Plugin operation failed", e);
        }
        return false;
    }

    private void handlePluginUndoCommand(JsonObject data) throws IOException {
        if (plugin.isActive() && plugin.supports(CopilotIDEPlugin.Commands.UNDO)) {
            List<String> files = listFiles(data);
            plugin.undo(files);
        }
    }

    private void handlePluginRedoCommand(JsonObject data) throws IOException {
        if (plugin.isActive() && plugin.supports(CopilotIDEPlugin.Commands.REDO)) {
            List<String> files = listFiles(data);
            plugin.redo(files);
        }
    }

    private void handlePluginRestartApplicationCommand() {
        if (plugin.isActive() && plugin.supports(CopilotIDEPlugin.Commands.RESTART_APPLICATION)) {
            plugin.restartApplication(getMainClass());
        }
    }

    private void handlePluginRefreshCommand() throws IOException {
        if (plugin.isActive()) {
            plugin.refresh();
        }
    }

    private List<String> listFiles(JsonObject data) {
        List<String> files = new ArrayList<>();
        files.addAll(JsonUtils.stream(data.getArray("files")).map(JsonValue::asString).map(this::filter).toList());
        if (data.hasKey(UI_ID)) {
            int flowUiId = (int) data.getNumber(UI_ID);
            if (flowUiId >= 0) {
                Map<ComponentTracker.Location, File> flowFiles = FlowUtil.findActiveJavaFiles(getVaadinSession(),
                        flowUiId);
                files.addAll(flowFiles.values().stream().map(File::getAbsolutePath).toList());
            }
        }
        return files;
    }

    private String filter(String filename) {
        if (filename.equals("$THEME_EDITOR_CSS")) {
            Optional<File> themeFolder = getProjectFileManager().getThemeFolder();
            if (themeFolder.isPresent()) {
                return themeFolder.get().toPath().resolve(ThemeModifier.THEME_EDITOR_CSS).toString();
            }
        }
        return filename;
    }

    private String getMainClass() {
        Class<?> appClass = SpringBridge.getApplicationClass(getVaadinContext());
        return appClass != null ? appClass.getName() : null;
    }
}

==> copilot-24.7.6-sources.jar: com/vaadin/copilot/ide/IdePluginCommandHandler.java <==
package com.vaadin.copilot.ide;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import com.vaadin.base.devserver.DevToolsInterface;
import com.vaadin.copilot.CopilotCommand;
import com.vaadin.copilot.SpringBridge;
import com.vaadin.copilot.plugins.themeeditor.ThemeModifier;
import com.vaadin.flow.internal.JsonUtils;

import elemental.json.Json;
import elemental.json.JsonObject;
import elemental.json.JsonValue;

public class IdePluginCommandHandler extends CopilotCommand {

    private final CopilotIDEPlugin plugin = CopilotIDEPlugin.getInstance();

    @Override
    public boolean handleMessage(String command, JsonObject data, DevToolsInterface devToolsInterface) {
        try {
            if (command.equals("plugin-undo")) {
                Optional<Boolean> maybePerformed = handlePluginUndoCommand(data).map(r -> r.getBoolean("performed"));
                JsonObject responseData = Json.createObject();
                responseData.put(KEY_REQ_ID, data.getString(KEY_REQ_ID));
                responseData.put("performed", maybePerformed.orElse(true));
                devToolsInterface.send("plugin-undo-response", responseData);
                return true;
            }

            if (command.equals("plugin-redo")) {
                Optional<Boolean> maybePerformed = handlePluginRedoCommand(data).map(r -> r.getBoolean("performed"));
                JsonObject responseData = Json.createObject();
                responseData.put(KEY_REQ_ID, data.getString(KEY_REQ_ID));
                responseData.put("performed", maybePerformed.orElse(true));
                devToolsInterface.send("plugin-redo-response", responseData);
                return true;
            }

            if (command.equals("plugin-restart-application")) {
                handlePluginRestartApplicationCommand();
                return true;
            }

            if (command.equals("plugin-refresh") && plugin.supports(CopilotIDEPlugin.Commands.REFRESH)) {
                handlePluginRefreshCommand();
                return true;
            }

        } catch (IOException e) {
            throw new UnsupportedOperationException("Copilot Plugin operation failed", e);
        }
        return false;
    }

    private Optional<JsonObject> handlePluginUndoCommand(JsonObject data) throws IOException {
        if (plugin.isActive() && plugin.supports(CopilotIDEPlugin.Commands.UNDO)) {
            List<String> files = listFiles(data);
            return plugin.undo(files);
        }
        return Optional.empty();
    }

    private Optional<JsonObject> handlePluginRedoCommand(JsonObject data) throws IOException {
        if (plugin.isActive() && plugin.supports(CopilotIDEPlugin.Commands.REDO)) {
            List<String> files = listFiles(data);
            return plugin.redo(files);
        }
        return Optional.empty();
    }

    private void handlePluginRestartApplicationCommand() {
        if (plugin.isActive() && plugin.supports(CopilotIDEPlugin.Commands.RESTART_APPLICATION)) {
            plugin.restartApplication(getMainClass());
        }
    }

    private void handlePluginRefreshCommand() throws IOException {
        if (plugin.isActive()) {
            plugin.refresh();
        }
    }

    private List<String> listFiles(JsonObject data) {
        List<String> files = new ArrayList<>();
        files.addAll(JsonUtils.stream(data.getArray("files")).map(JsonValue::asString).map(this::filter).toList());
        return files;
    }

    private String filter(String filename) {
        if (filename.equals("$THEME_EDITOR_CSS")) {
            Optional<File> themeFolder = getProjectFileManager().getThemeFolder();
            if (themeFolder.isPresent()) {
                return themeFolder.get().toPath().resolve(ThemeModifier.THEME_EDITOR_CSS).toString();
            }
        }
        return filename;
    }

    private String getMainClass() {
        Class<?> appClass = SpringBridge.getApplicationClass(getVaadinContext());
        return appClass != null ? appClass.getName() : null;
    }
}
